@@ -7,7 +7,8 @@ from applitools.common import RectangleSize, Region, logger
 from applitools.common.utils import image_utils
 from applitools.core.capture import ImageProvider
 from applitools.selenium.positioning import ScrollPositionProvider
-from applitools.selenium.useragent import BrowserNames, OSNames, UserAgent
+from applitools.selenium.useragent import BrowserNames, UserAgent
+from applitools.selenium.viewport_locator import device_viewport_location
 
 if typing.TYPE_CHECKING:
     from PIL.Image import Image
@@ -49,7 +50,6 @@ class DeviceData(object):
 @attr.s
 class MobileSafariScreenshotImageProvider(ImageProvider):
     _devices_regions = None
-
     _eyes = attr.ib()  # type: SeleniumEyes
     _useragent = attr.ib()  # type: UserAgent
 
@@ -59,37 +59,49 @@ class MobileSafariScreenshotImageProvider(ImageProvider):
         if self._eyes.is_cut_provider_explicitly_set:
             return image
 
-        # original_viewport_size = self._eyes._get_viewport_size()
-        # FIXME: huck to get correct viewport size on mobile
-        original_viewport_size = RectangleSize.from_(
-            self._eyes.scroll_root_element.size_and_borders.size
-        )
-        logger.info("logical viewport size: {}".format(original_viewport_size))
-
-        if self._devices_regions is None:
-            self.init_device_regions_table()
-
         logger.info(
-            "physical device pixel size: {} x {}".format(image.width, image.height)
+            "physical device pixel size", width=image.width, height=image.height
         )
 
-        device_data = DeviceData(
-            image.width,
-            image.height,
-            original_viewport_size.width,
-            original_viewport_size.height,
-        )
-        if device_data in self._devices_regions:
-            logger.debug("device model found in hash table")
-            crop_by_version = self._devices_regions.get(device_data)
-            if crop_by_version.major_version <= self._useragent.browser_major_version:
-                image = image_utils.crop_image(image, crop_by_version.region)
-            else:
-                logger.debug(
-                    "device version not found in list. returning original image."
-                )
+        viewport_location = device_viewport_location(self._eyes.driver)
+        if viewport_location:
+            original_viewport_size = self._eyes._get_viewport_size()
+            viewport_size = original_viewport_size.scale(self._eyes.device_pixel_ratio)
+            crop_region = Region(
+                viewport_location.x,
+                viewport_location.y,
+                viewport_size.width,
+                viewport_size.height,
+            )
+            logger.info("calculated viewport region", viewport_region=crop_region)
+            image = image_utils.crop_image(image, crop_region)
         else:
-            logger.debug("device not found in list. returning original image.")
+            if self._devices_regions is None:
+                self.init_device_regions_table()
+            original_viewport_size = RectangleSize.from_(
+                self._eyes.scroll_root_element.size_and_borders.size
+            )
+            device_data = DeviceData(
+                image.width,
+                image.height,
+                original_viewport_size.width,
+                original_viewport_size.height,
+            )
+            if device_data in self._devices_regions:
+                logger.debug("device model found in hash table")
+                crop_by_version = self._devices_regions.get(device_data)
+                if (
+                    crop_by_version.major_version
+                    <= self._useragent.browser_major_version
+                ):
+                    image = image_utils.crop_image(image, crop_by_version.region)
+                else:
+                    logger.debug(
+                        "device version not found in list. returning original image."
+                    )
+            else:
+                logger.debug("device not found in list. returning original image.")
+
         return image
 
     def init_device_regions_table(self):
@@ -99,7 +111,8 @@ class MobileSafariScreenshotImageProvider(ImageProvider):
             DeviceData(1792, 828, 808, 364): r_v(12, Region(88, 101, 1616, 685)),
             DeviceData(1242, 2688, 414, 719): r_v(12, Region(0, 283, 1242, 2155)),
             DeviceData(2688, 1242, 808, 364): r_v(12, Region(132, 151, 2424, 1028)),
-            DeviceData(1125, 2436, 375, 635): r_v(11, Region(0, 283, 1125, 1903)),
+            DeviceData(1125, 2436, 375, 635): r_v(11, Region(0, 282, 1125, 1905)),
+            DeviceData(1125, 2436, 980, 1659): r_v(13, Region(0, 282, 1125, 1905)),
             DeviceData(2436, 1125, 724, 325): r_v(11, Region(132, 151, 2436, 930)),
             DeviceData(1242, 2208, 414, 622): r_v(11, Region(0, 211, 1242, 1863)),
             DeviceData(2208, 1242, 736, 364): r_v(11, Region(0, 151, 2208, 1090)),
@@ -128,7 +141,6 @@ class MobileSafariScreenshotImageProvider(ImageProvider):
 
 @attr.s
 class SafariScreenshotImageProvider(ImageProvider):
-
     _eyes = attr.ib()  # type: SeleniumEyes
     _useragent = attr.ib()  # type: UserAgent
 
