@@ -14,7 +14,7 @@ from applitools.common import (
     logger,
 )
 from applitools.common.geometry import Point
-from applitools.common.selenium import StitchMode
+from applitools.common.selenium import Configuration, StitchMode
 from applitools.common.utils import argument_guard, datetime_utils, image_utils
 from applitools.core import (
     NULL_REGION_PROVIDER,
@@ -49,7 +49,6 @@ from .region_compensation import (
     RegionPositionCompensation,
     get_region_position_compensation,
 )
-from .useragent import BrowserNames
 from .webdriver import EyesWebDriver
 from .webelement import EyesWebElement, adapt_element
 
@@ -89,7 +88,7 @@ class SeleniumEyes(EyesBase):
     _element_position_provider = None  # type: Optional[ElementPositionProvider]
     _check_frame_or_element = None  # type: bool
     _original_fc = None  # type: Optional[FrameChain]
-    _scroll_root_element = None  # type: Optional[EyesWebElement]
+    _user_defined_SRE = None  # type: Optional[EyesWebElement]
     _effective_viewport = None  # type: Optional[Region]
     _target_element = None  # type: Optional[EyesWebElement]
     _screenshot_factory = None  # type: Optional[EyesWebDriverScreenshotFactory]
@@ -125,7 +124,12 @@ class SeleniumEyes(EyesBase):
         self._do_not_get_title = False
         self._device_pixel_ratio = self._UNKNOWN_DEVICE_PIXEL_RATIO
         self._stitch_content = False  # type: bool
-        self._runner = runner if runner else ClassicRunner()
+        self._runner = runner or ClassicRunner()
+
+    @property
+    def configure(self):
+        # type:() -> Configuration
+        return super(SeleniumEyes, self).configure
 
     @property
     def original_fc(self):
@@ -222,7 +226,7 @@ class SeleniumEyes(EyesBase):
         return False
 
     def _create_position_provider(self, scroll_root_element):
-
+        # type: (AnyWebElement) -> PositionProvider
         return create_position_provider(
             self.driver,
             self.configure.stitch_mode,
@@ -230,8 +234,8 @@ class SeleniumEyes(EyesBase):
             self._target_element,
         )
 
-    def _match_window(self, region_provider, tag, check_settings, source):
-        # type:(RegionProvider,Text,SeleniumCheckSettings,Optional[Text])->MatchResult
+    def _match_window(self, region_provider, check_settings, source):
+        # type:(RegionProvider,SeleniumCheckSettings,Optional[Text])->MatchResult
         if check_settings.values.ocr_region:
             check_settings.values.ocr_region.process_app_output(
                 check_settings, region_provider.get_region()
@@ -239,11 +243,11 @@ class SeleniumEyes(EyesBase):
             return MatchResult(as_expected=True)
 
         return super(SeleniumEyes, self)._match_window(
-            region_provider, tag, check_settings, source
+            region_provider, check_settings, source
         )
 
     def check(self, check_settings):
-        # type: (Text, SeleniumCheckSettings) -> MatchResult
+        # type: (SeleniumCheckSettings) -> MatchResult
         source = eyes_selenium_utils.get_check_source(self.driver)
         name = check_settings.values.name
 
@@ -251,12 +255,10 @@ class SeleniumEyes(EyesBase):
 
         # Set up required settings
         self._stitch_content = check_settings.values.stitch_content
-        self._scroll_root_element = eyes_selenium_utils.scroll_root_element_from(
+        self._user_defined_SRE = eyes_selenium_utils.scroll_root_element_from(
             self.driver, check_settings
         )
-        self._position_provider = self._create_position_provider(
-            self._scroll_root_element
-        )
+        self._position_provider = self._create_position_provider(self._user_defined_SRE)
 
         self._original_fc = self.driver.frame_chain.clone()
 
@@ -270,18 +272,18 @@ class SeleniumEyes(EyesBase):
 
             logger.info("Current URL: {}".format(self._driver.current_url))
             with self._switch_to_frame(check_settings):
-                result = self._check_result_flow(name, check_settings, source)
+                result = self._check_result_flow(check_settings, source)
 
             # restore scrollbar of main window
-            self._scroll_root_element = eyes_selenium_utils.scroll_root_element_from(
+            self._user_defined_SRE = eyes_selenium_utils.scroll_root_element_from(
                 self.driver, check_settings
             )
             self._try_restore_scrollbars()
         else:
-            result = self._check_result_flow(name, check_settings, source)
+            result = self._check_result_flow(check_settings, source)
 
         self._stitch_content = False
-        self._scroll_root_element = None
+        self._user_defined_SRE = None
         if self._position_memento:
             ScrollPositionProvider(self.driver, self.scroll_root_element).restore_state(
                 self._position_memento
@@ -304,7 +306,7 @@ class SeleniumEyes(EyesBase):
         self._screenshot_factory = None
         return results
 
-    def _check_result_flow(self, name, check_settings, source):
+    def _check_result_flow(self, check_settings, source):
         target_region = check_settings.values.target_region
         result = None
         if target_region and self._switched_to_frame_count == 0:
@@ -312,7 +314,7 @@ class SeleniumEyes(EyesBase):
             target_region = target_region.clone()
             target_region.coordinates_type = CoordinatesType.CONTEXT_RELATIVE
             result = self._check_window_base(
-                RegionProvider(target_region), name, False, check_settings, source
+                RegionProvider(target_region), False, check_settings, source
             )
         elif check_settings:
             target_element = self._element_from(check_settings)
@@ -325,40 +327,49 @@ class SeleniumEyes(EyesBase):
                 logger.debug("have target element")
                 self._target_element = target_element
                 if self._stitch_content:
-                    result = self._check_element(name, check_settings, source)
+                    result = self._check_full_element(check_settings, source)
                 else:
-                    result = self._check_region(name, check_settings, source)
+                    result = self._check_element(check_settings, source)
                 self._target_element = None
             elif total_frames > 0:
                 logger.debug("have frame chain")
                 if self._stitch_content:
-                    result = self._check_full_frame_or_element(
-                        name, check_settings, source
-                    )
+                    result = self._check_full_frame(check_settings, source)
                 else:
-                    result = self._check_frame_fluent(name, check_settings, source)
-            else:
-                logger.debug("default case")
-                if not self.driver.is_mobile_app:
-                    # required to prevent cut line on the last stitched part of the
-                    # page on some browsers (like firefox).
-                    self.driver.switch_to.default_content()
-                    self._current_frame_position_provider = (
-                        self._create_position_provider(
-                            self.driver.find_element_by_tag_name("html")
-                        )
+                    logger.debug("Target.Frame(frame).Fully(false)")
+                    logger.debug(
+                        "WARNING: This shouldn't have been called, as it is covered "
+                        "by `check_element(...)` "
                     )
-                result = self._check_window_base(
-                    NULL_REGION_PROVIDER,
-                    name,
-                    False,
-                    check_settings,
-                    source,
-                )
+                    result = self._check_frame(check_settings, source)
+            else:
+                if self._stitch_content:
+                    self._check_full_window(check_settings, source)
+                else:
+                    self._check_window(check_settings, source)
+
         if result is None:
             result = MatchResult()
         return result
 
+    def _check_full_window(self, check_settings, source):
+        logger.debug("Target.window().fully()")
+        return self._check_window_base(
+            NULL_REGION_PROVIDER,
+            False,
+            check_settings,
+            source,
+        )
+
+    def _check_window(self, check_settings, source):
+        logger.debug("Target.window()")
+        return self._check_window_base(
+            NULL_REGION_PROVIDER,
+            False,
+            check_settings,
+            source,
+        )
+
     @property
     def current_frame_position_provider(self):
         if self._current_frame_position_provider:
@@ -366,7 +377,7 @@ class SeleniumEyes(EyesBase):
         else:
             return self.position_provider
 
-    def _check_full_frame_or_element(self, name, check_settings, source):
+    def _check_full_frame(self, check_settings, source):
         self._check_frame_or_element = True
 
         def full_frame_or_element_region(check_settings):
@@ -374,7 +385,8 @@ class SeleniumEyes(EyesBase):
                 "check_frame_or_element: {}".format(self._check_frame_or_element)
             )
             if self._check_frame_or_element:
-                fc = self._ensure_frame_visible()
+                self._ensure_frame_visible()
+                fc = self.driver.frame_chain.clone()
                 # FIXME - Scaling should be handled in a single place instead
                 scale_factory = self.update_scaling_params()
                 screenshot_image = self._image_provider.get_image()
@@ -395,7 +407,6 @@ class SeleniumEyes(EyesBase):
 
         result = self._check_window_base(
             RegionProvider(lambda: full_frame_or_element_region(check_settings)),
-            name,
             False,
             check_settings,
             source,
@@ -404,21 +415,21 @@ class SeleniumEyes(EyesBase):
         self._region_to_check = None
         return result
 
-    def _check_frame_fluent(self, name, check_settings, source):
+    def _check_frame(self, check_settings, source):
         fc = self.driver.frame_chain.clone()
         target_frame = fc.pop()
         self._target_element = target_frame.reference
 
         self.driver.switch_to.frames_do_scroll(fc)
-        result = self._check_region(name, check_settings, source)
+        result = self._check_element(check_settings, source)
         self._target_element = None
         return result
 
-    def _check_element(self, name, check_settings, source):
+    def _check_full_element(self, check_settings, source):
         element = self._target_element  # type: EyesWebElement
 
         scroll_root_element = eyes_selenium_utils.curr_frame_scroll_root_element(
-            self.driver, self._scroll_root_element
+            self.driver, self._user_defined_SRE
         )
         pos_provider = self._create_position_provider(scroll_root_element)
 
@@ -428,7 +439,7 @@ class SeleniumEyes(EyesBase):
         result = None
         with eyes_selenium_utils.get_and_restore_state(pos_provider):
             with self._ensure_element_visible(element):
-                pl = element.location
+                pl = Point.from_(element.bounding_client_rect)
                 try:
                     self._check_frame_or_element = True
                     display_style = element.get_computed_style("display")
@@ -481,7 +492,6 @@ class SeleniumEyes(EyesBase):
 
                     result = self._check_window_base(
                         NULL_REGION_PROVIDER,
-                        name,
                         False,
                         check_settings,
                         source,
@@ -498,22 +508,25 @@ class SeleniumEyes(EyesBase):
                     self._full_region_to_check = None
         return result
 
-    def _check_region(self, name, check_settings, source):
+    def _check_element(self, check_settings, source):
         self._is_check_region = True
 
         def get_region():
             rect = check_settings.values.target_region
             if rect is None:
-                if self.driver.is_mobile_platform:
-                    bounds = self._target_element.rect
-                else:
-                    bounds = self._target_element.bounding_client_rect
+                # Make sure bounds are calculated with the same scrolling
+                # positions as when screenshot is being made
+                with self._ensure_element_visible(self._target_element):
+                    if self.driver.is_mobile_app:
+                        bounds = self._target_element.rect
+                    else:
+                        bounds = self._target_element.bounding_client_rect
                 region = Region(
                     bounds["x"],
                     bounds["y"],
                     bounds["width"],
                     bounds["height"],
-                    coordinates_type=CoordinatesType.CONTEXT_RELATIVE,
+                    coordinates_type=CoordinatesType.CONTEXT_AS_IS,
                 )
             else:
                 s = self._target_element.size_and_borders.size
@@ -530,7 +543,6 @@ class SeleniumEyes(EyesBase):
 
         result = self._check_window_base(
             RegionProvider(get_region),
-            name,
             False,
             check_settings,
             source,
@@ -539,7 +551,7 @@ class SeleniumEyes(EyesBase):
         return result
 
     def _ensure_frame_visible(self):
-        logger.debug("scroll_root_element_: {}".format(self._scroll_root_element))
+        logger.debug("scroll_root_element_: {}".format(self._user_defined_SRE))
         current_fc = self.driver.frame_chain.clone()
         if not current_fc:
             # if no frames no point to go below
@@ -547,6 +559,7 @@ class SeleniumEyes(EyesBase):
         fc = self.driver.frame_chain.clone()
         self.driver.execute_script("window.scrollTo(0,0);")
         origin_driver = eyes_selenium_utils.get_underlying_driver(self.driver)
+        restore_scroll_stack = []
 
         while len(fc) > 0:
             logger.debug("fc count: {}".format(fc.size))
@@ -558,7 +571,7 @@ class SeleniumEyes(EyesBase):
             if fc.size == self._original_fc.size:
                 logger.debug("PositionProvider: {}".format(self.position_provider))
                 self._position_memento = self.position_provider.get_state()
-                scroll_root_element = self._scroll_root_element
+                scroll_root_element = self._user_defined_SRE
             else:
                 if parent_frame:
                     scroll_root_element = parent_frame.scroll_root_element
@@ -569,11 +582,25 @@ class SeleniumEyes(EyesBase):
             position_provider = self._element_position_provider_from(
                 scroll_root_element
             )
+            restore_scroll_stack.append(
+                (
+                    position_provider,
+                    position_provider.get_state(),
+                    child_frame.reference,
+                )
+            )
             position_provider.set_position(child_frame.location)
             reg = Region.from_(Point.ZERO(), child_frame.inner_size)
             self._effective_viewport = self._effective_viewport.intersect(reg)
         self.driver.switch_to.frames(current_fc)
-        return current_fc
+        return restore_scroll_stack
+
+    def _restore_framechain_scroll(self, restore_stack):
+        with self.driver.saved_frame_chain():
+            self.driver.switch_to.default_content()
+            for position_provider, state, child_frame_ref in reversed(restore_stack):
+                position_provider.restore_state(state)
+                self.driver.switch_to.frame(child_frame_ref)
 
     def _element_position_provider_from(self, scroll_root_element):
         # type: (EyesWebElement) -> PositionProvider
@@ -589,7 +616,6 @@ class SeleniumEyes(EyesBase):
     def _switch_to_frame(self, check_settings):
         # type: (SeleniumCheckSettings) -> Generator
         self._switched_to_frame_count = 0
-        # TODO: refactor frames storing
         frames = {}
         frame_chain = check_settings.values.frame_chain
         for frame_locator in frame_chain:
@@ -603,6 +629,11 @@ class SeleniumEyes(EyesBase):
             frames[self._switched_to_frame_count] = cur_frame
             self._switched_to_frame_count += 1
 
+        # if already switched into frame
+        # assign user defined scroll root element into latest frame #Trello2136
+        if not frame_chain and self.driver.frame_chain and self._user_defined_SRE:
+            self.driver.frame_chain.peek.scroll_root_element = self._user_defined_SRE
+
         yield
 
         while self._switched_to_frame_count > 0:
@@ -615,9 +646,9 @@ class SeleniumEyes(EyesBase):
 
     @property
     def scroll_root_element(self):
-        if self._scroll_root_element is None:
-            self._scroll_root_element = self.driver.find_element_by_tag_name("html")
-        return self._scroll_root_element
+        if self._user_defined_SRE is None:
+            self._user_defined_SRE = self.driver.find_element_by_tag_name("html")
+        return self._user_defined_SRE
 
     def add_mouse_trigger_by_element(self, action, element):
         # type: (Text, AnyWebElement) -> None
@@ -805,7 +836,9 @@ class SeleniumEyes(EyesBase):
             # and mislead users.
             return None
         try:
-            dom_json = dom_capture.get_full_window_dom(self._driver)
+            dom_json = dom_capture.get_full_window_dom(
+                self._driver, self.server_connector, False
+            )
             return dom_json
         except Exception:
             logger.warning("dom-capture script failed, skipping it", exc_info=True)
@@ -834,7 +867,7 @@ class SeleniumEyes(EyesBase):
 
     def _create_full_page_capture_algorithm(self, scale_provider):
         scroll_root_element = eyes_selenium_utils.curr_frame_scroll_root_element(
-            self.driver, self._scroll_root_element
+            self.driver, self._user_defined_SRE
         )
         origin_provider = ScrollPositionProvider(self.driver, scroll_root_element)
         return FullPageCaptureAlgorithm(
@@ -1002,6 +1035,7 @@ class SeleniumEyes(EyesBase):
     @contextlib.contextmanager
     def _ensure_element_visible(self, element):
         position_provider = fc = None
+        restore_frame_scroll_stack = []
         if element and not self.driver.is_mobile_app:
             original_fc = self.driver.frame_chain.clone()
             element_bounds = element.bounds
@@ -1016,14 +1050,14 @@ class SeleniumEyes(EyesBase):
             )
 
             if not viewport_bounds.contains(element_bounds):
-                self._ensure_frame_visible()
+                restore_frame_scroll_stack = self._ensure_frame_visible()
                 element_location = Point.from_(element.location)
                 if len(original_fc) > 0 and element is not original_fc.peek.reference:
                     fc = original_fc
                     self.driver.switch_to.frames(original_fc)
                     scroll_root_element = (
                         eyes_selenium_utils.curr_frame_scroll_root_element(
-                            self.driver, self._scroll_root_element
+                            self.driver, self._user_defined_SRE
                         )
                     )
                 # This might happen when scroll root element is calculated in the
@@ -1042,6 +1076,9 @@ class SeleniumEyes(EyesBase):
                 position_provider.set_position(element_location)
 
         yield position_provider
-        if element and position_provider and fc and not self.driver.is_mobile_app:
-            self.driver.switch_to.frames(fc)
-            position_provider.restore_state(state)
+        if not self.driver.is_mobile_app:
+            if element and position_provider:
+                if fc:
+                    self.driver.switch_to.frames(fc)
+                position_provider.restore_state(state)
+            self._restore_framechain_scroll(restore_frame_scroll_stack)
