@@ -48,6 +48,7 @@ class EyesConnector(EyesBase):
         browser_info,  # type: RenderBrowserInfo
         config,  # type: Configuration
         server_connector,  # type: ServerConnector
+        agent_run_id,  # type: Text
         job_info=None,  # type: Optional[JobInfo]
     ):
         # type: (...) -> None
@@ -61,6 +62,7 @@ class EyesConnector(EyesBase):
         self._region_selectors = None
         self._regions = None
         self._job_info = job_info  # type: Optional[JobInfo]
+        self._agent_run_id = agent_run_id
 
     def open(self, config):
         # type: (Configuration) -> None
@@ -87,9 +89,9 @@ class EyesConnector(EyesBase):
         # type: (*Text) -> List[RenderStatusResults]
         return self._server_connector.render_status_by_id(*render_ids)
 
-    def download_resource(self, url):
-        # type: (Text) -> Response
-        return self._server_connector.download_resource(url)
+    def download_resource(self, url, cookies):
+        # type: (Text, Dict) -> Response
+        return self._server_connector.download_resource(url, cookies)
 
     @property
     def base_agent_id(self):
@@ -178,7 +180,7 @@ class EyesConnector(EyesBase):
         self._region_selectors = region_selectors
         self._regions = regions
         check_result = self._check_window_base(
-            NULL_REGION_PROVIDER, name, False, check_settings, source
+            NULL_REGION_PROVIDER, False, check_settings, source
         )
         self._current_uuid = None
         self._region_selectors = []
@@ -209,15 +211,19 @@ class EyesConnector(EyesBase):
             )
         return status
 
-    def _match_window(self, region_provider, tag, check_settings, source):
-        # type: (RegionProvider, Text, SeleniumCheckSettings, Optional[Text]) -> MatchResult
+    def _match_window(self, region_provider, check_settings, source):
+        # type: (RegionProvider, SeleniumCheckSettings, Optional[Text]) -> MatchResult
         # Update retry timeout if it wasn't specified.
         retry_timeout_ms = -1  # type: int
         if check_settings:
             retry_timeout_ms = check_settings.values.timeout
 
         region = region_provider.get_region()
-        logger.debug("params: ([{}], {}, {} ms)".format(region, tag, retry_timeout_ms))
+        logger.debug(
+            "params: ([{}], {}, {} ms)".format(
+                region, check_settings.values.name, retry_timeout_ms
+            )
+        )
 
         app_output = self._get_app_output_with_screenshot(None, None, check_settings)
         image_match_settings = self._match_window_task.create_image_match_settings(
@@ -226,7 +232,6 @@ class EyesConnector(EyesBase):
         return self._match_window_task.perform_match(
             app_output=app_output,
             replace_last=False,
-            name=tag,
             image_match_settings=image_match_settings,
             eyes=self,
             user_inputs=self._user_inputs,
